from plantstar_shared import syscon_json

# Converts bytes into a decoded string, then into an object using syscon_json.loads
def convert_bytes_to_object(object_as_encoded_string, decoding_format="ASCII"):
    object_as_decoded_string = object_as_encoded_string.decode(decoding_format)
    output_object = syscon_json.loads(object_as_decoded_string)

    return output_object



from plantstar_shared.RawDataProcessorInterfaceActions import RawDataProcessorInterfaceActions
from plantstar_shared.errors import SysconProgrammingError


def obtain_raw_data_processor_function_for_action_name(rawdataprocessor, action_name):
    if action_name == RawDataProcessorInterfaceActions.GET_DRIVER_VERSION_STRING:
        return rawdataprocessor.get_driver_version_string
    elif action_name == RawDataProcessorInterfaceActions.GET_INPUT_SIGNAL_DICTIONARIES:
        return rawdataprocessor.get_input_signal_dictionaries
    elif action_name == RawDataProcessorInterfaceActions.GET_VERSION_STRING:
        return rawdataprocessor.get_version_string
    elif action_name == RawDataProcessorInterfaceActions.UPDATE_ANALOG_INPUTS:
        return rawdataprocessor.update_analog_inputs
    elif action_name == RawDataProcessorInterfaceActions.UPDATE_DIGITAL_INPUT_DEBOUNCE:
        return rawdataprocessor.update_digital_input_debounce
    elif action_name == RawDataProcessorInterfaceActions.UPLOAD_NEW_DRIVER_SETTINGS:
        return rawdataprocessor.upload_new_driver_settings
    else:
        raise SysconProgrammingError



import struct

# Creates a message that is comprised of the size of an encoded string + that encoded string for use in sending messages over TCP sockets
def add_size_onto_string_and_return(encoded_string):
    size_of_string = len(encoded_string)
    encoded_string_prepended_with_size = struct.pack('>I', size_of_string) + encoded_string
    return encoded_string_prepended_with_size



from plantstar_shared.SysconType import SysconType


# `None` tells the system to use the real status retrieved, where a `bool` value is used to signify that we want to mock the status of some process: `is_running_mock_status`
# Local dev systems do not use this, as the API call is entirely skipped on the APU
# The first group represents Live VM Systems
# The second group represents Live (Real) Systems

class DataCollectionModuleProcessNames(SysconType):
    PROCESS_SPAWNER = ("Process Spawner", None, None)
    RAW_DATA_PROCESSOR_INTERFACE = ("Raw Data Processor Interface", None, None)
    RTL_RIO_3 = ("RTL RIO 3", True, None)



import setuptools

with open("README.md", "r", encoding="utf-8") as file_handle:
    long_description = file_handle.read()

setuptools.setup(
    name="plantstar-shared",
    version="0.1.19",
    author="SYSCON International",
    author_email="dev@syscon-intl.com",
    description="Shared code used in plantstar_apu and plantstar_dcm",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/SYSCON-International/plantstar_shared",
    packages=setuptools.find_packages(),
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
    install_requires=[
        # Make sure to update the requirements.txt to match any changes made to this section
        "Django>=4.0.4",
        "psycopg2>=2.9.3",
        "python-dateutil>=2.8.2",
        "pytz>=2022.1",
        "requests>=2.27.1",
    ]
)



class InvalidApiRequest(Exception):
    pass


class SysconProgrammingError(Exception):
    pass

    @staticmethod
    def get_invalid_type_error(the_type, invalid_type_supplied):
        return SysconProgrammingError(f"Invalid \"{the_type.__name__}\" type: \"{invalid_type_supplied}\"")



from django.core.files.images import ImageFile
from django.db.models.fields.files import FieldFile, ImageField


class SysconImageFieldFile(ImageFile, FieldFile):
    @property
    def src_path(self):
        if self:
            if str(self).startswith("http"):
                return str(self)
            else:
                return self.url

        return ""


class SysconImageField(ImageField):
    attr_class = SysconImageFieldFile



from plantstar_shared import syscon_json

# Converts the output dictionary into a string through syscon_json.dumps, then encodes it into bytes so that it can be sent through the socket, and parsed later on
def convert_object_to_bytes(input_object, encoding_format="ASCII"):
    input_object_as_string = syscon_json.dumps(input_object)
    input_object_as_bytes = input_object_as_string.encode(encoding_format)

    return input_object_as_bytes



import struct

from plantstar_shared.add_size_onto_string_and_return import add_size_onto_string_and_return
from plantstar_shared.convert_bytes_to_object import convert_bytes_to_object
from plantstar_shared.convert_object_to_bytes import convert_object_to_bytes


SIZE_OF_UNSIGNED_INT_STRUCT = 4


class SocketConnectionError(Exception):
    pass


def read_size_value_from_socket(*, remote_socket):
    size_struct = remote_socket.recv(SIZE_OF_UNSIGNED_INT_STRUCT)
    
    if not size_struct:
        raise SocketConnectionError

    try:
        message_length = struct.unpack('>I', size_struct)[0]
    except struct.error as error:
        raise SocketConnectionError

    return message_length


def get_message_from_socket(*, remote_socket):
    message_length = read_size_value_from_socket(remote_socket=remote_socket)

    data = bytearray()

    # While loop that will stream in data if the full request is not available yet
    while len(data) < message_length:
        packet = remote_socket.recv(message_length - len(data))

        if not packet:
            return None

        data.extend(packet)
    return data


def send_message_on_socket(*, remote_socket, dumpsable_object, encoding='ASCII'):
    encoded_message = convert_object_to_bytes(input_object=dumpsable_object, encoding_format=encoding)
    send_encoded_message_on_socket(remote_socket=remote_socket, encoded_message=encoded_message)


def send_encoded_message_on_socket(*, remote_socket, encoded_message):
    encoded_message_with_size = add_size_onto_string_and_return(encoded_message)
    remote_socket.send(encoded_message_with_size)


def get_object_from_socket(*, remote_socket):
    object_from_interface_as_bytes = get_message_from_socket(remote_socket=remote_socket)
    object_from_interface = convert_bytes_to_object(object_from_interface_as_bytes)
    return object_from_interface
